@@ -9,6 +9,7 @@ import re
 import subprocess
 import sys
 from functools import lru_cache
+from importlib import import_module
 from pathlib import Path
 from typing import Callable, Sequence
 
@@ -17,6 +18,7 @@ import jpype.config
 from jgo import jgo
 
 import scyjava.config
+from scyjava.config import Mode, mode
 
 _logger = logging.getLogger(__name__)
 
@@ -78,28 +80,34 @@ class JavaClasses:
 
 def jvm_version() -> str:
     """
-    Gets the version of the JVM as a tuple,
-    with each dot-separated digit as one element.
-    Characters in the version string beyond only
-    numbers and dots are ignored, in line
-    with the java.version system property.
+    Gets the version of the JVM as a tuple, with each dot-separated digit
+    as one element. Characters in the version string beyond only numbers
+    and dots are ignored, in line with the java.version system property.
 
     Examples:
     * OpenJDK 17.0.1 -> [17, 0, 1]
     * OpenJDK 11.0.9.1-internal -> [11, 0, 9, 1]
     * OpenJDK 1.8.0_312 -> [1, 8, 0]
 
-    If the JVM is already started,
-    this function should return the equivalent of:
+    If the JVM is already started, this function returns the equivalent of:
        jimport('java.lang.System')
          .getProperty('java.version')
          .split('.')
 
-    In case the JVM is not started yet,a best effort is made to deduce
+    In case the JVM is not started yet, a best effort is made to deduce
     the version from the environment without actually starting up the
     JVM in-process. If the version cannot be deduced, a RuntimeError
     with the cause is raised.
     """
+    if mode == Mode.JEP:
+        System = jimport("java.lang.System")
+        version = str(System.getProperty("java.version"))
+        # Get everything up to the hyphen
+        version = version.split("-")[0]
+        return tuple(map(int, version.split(".")))
+
+    assert mode == Mode.JPYPE
+
     jvm_version = jpype.getJVMVersion()
     if jvm_version and jvm_version[0]:
         # JPype already knew the version.
@@ -108,8 +116,7 @@ def jvm_version() -> str:
         return jvm_version
 
     # JPype was clueless, which means the JVM has probably not started yet.
-    # Let's look for a java executable, and ask it directly with 'java
-    # -version'.
+    # Let's look for a java executable, and ask via 'java -version'.
 
     default_jvm_path = jpype.getDefaultJVMPath()
     if not default_jvm_path:
@@ -137,12 +144,16 @@ def jvm_version() -> str:
     if java is None:
         raise RuntimeError(f"No java executable found inside: {p}")
 
-    version = subprocess.check_output(
-        [str(java), "-version"], stderr=subprocess.STDOUT
-    ).decode()
-    m = re.match('.*version "(([0-9]+\\.)+[0-9]+)', version)
+    try:
+        output = subprocess.check_output(
+            [str(java), "-version"], stderr=subprocess.STDOUT
+        ).decode()
+    except subprocess.CalledProcessError as e:
+        raise RuntimeError("System call to java failed") from e
+
+    m = re.match('.*version "(([0-9]+\\.)+[0-9]+)', output)
     if not m:
-        raise RuntimeError(f"Inscrutable java command output:\n{version}")
+        raise RuntimeError(f"Inscrutable java command output:\n{output}")
 
     return tuple(map(int, m.group(1).split(".")))
 
@@ -163,6 +174,8 @@ def start_jvm(options=None) -> None:
         _logger.debug("The JVM is already running.")
         return
 
+    assert mode == Mode.JPYPE
+
     # retrieve endpoint and repositories from scyjava config
     endpoints = scyjava.config.endpoints
     repositories = scyjava.config.get_repositories()
@@ -191,7 +204,6 @@ def start_jvm(options=None) -> None:
         or not os.environ["JAVA_HOME"]
         or not os.path.isdir(os.environ["JAVA_HOME"])
     ):
-
         _logger.debug("JAVA_HOME not set. Will try to infer it from sys.path.")
 
         libjvm_win = Path("Library") / "jre" / "bin" / "server" / "jvm.dll"
@@ -249,10 +261,17 @@ def shutdown_jvm() -> None:
 
     Note that if the JVM is not already running, then this function does
     nothing! In particular, shutdown hooks are skipped in this situation.
+
+    :raises RuntimeError: if this method is called while in Jep mode.
     """
     if not jvm_started():
         return
 
+    if mode == Mode.JEP:
+        raise RuntimeError("Cannot shut down the JVM in Jep mode.")
+
+    assert mode == Mode.JPYPE
+
     # invoke registered shutdown callback functions
     for callback in _shutdown_callbacks:
         try:
@@ -275,6 +294,11 @@ def shutdown_jvm() -> None:
 
 def jvm_started() -> bool:
     """Return true iff a Java virtual machine (JVM) has been started."""
+    if mode == Mode.JEP:
+        return True
+
+    assert mode == Mode.JPYPE
+
     return jpype.isJVMStarted()
 
 
@@ -346,11 +370,19 @@ def when_jvm_stops(f) -> None:
 
 def isjava(data) -> bool:
     """Return whether the given data object is a Java object."""
+    if mode == Mode.JEP:
+        return jinstance(data, "java.lang.Object")
+
+    assert mode == Mode.JPYPE
     return isinstance(data, jpype.JClass) or isinstance(data, jpype.JObject)
 
 
 def is_jarray(data) -> bool:
     """Return whether the given data object is a Java array."""
+    if mode == Mode.JEP:
+        return str(type(data)) == "<class 'jep.PyJArray'>"
+
+    assert mode == Mode.JPYPE
     return isinstance(data, jpype.JArray)
 
 
@@ -363,6 +395,12 @@ def jimport(class_name: str):
     :returns: A pointer to the class, which can be used to
               e.g. instantiate objects of that class.
     """
+    if mode == Mode.JEP:
+        module_path = class_name.rsplit(".", 1)
+        module = import_module(module_path[0], module_path[1])
+        return getattr(module, module_path[1])
+
+    assert mode == Mode.JPYPE
     start_jvm()
     return jpype.JClass(class_name)
 
@@ -371,21 +409,45 @@ def jclass(data):
     """
     Obtain a Java class object.
 
-    :param data: The object from which to glean the class.
     Supported types include:
-    A. Name of a class to look up, analogous to
-    Class.forName("java.lang.String");
-    B. A jpype.JClass object analogous to String.class;
-    C. A jpype.JObject instance analogous to o.getClass().
+
+    A. Name of a class to look up -- e.g. "java.lang.String" --
+       which returns the equivalent of Class.forName("java.lang.String").
+
+    B. A static-style class reference -- e.g. String --
+       which returns the equivalent of String.class.
+
+    C. A Java object -- e.g. foo --
+       which returns the equivalent of foo.getClass().
+
+    Note that if you pass a java.lang.Class object, you will get back Class.class,
+    i.e. the Java class for the Class class. :-)
+
+    :param data: The object from which to glean the class.
     :returns: A java.lang.Class object, suitable for use with reflection.
     :raises TypeError: if the argument is not one of the aforementioned types.
     """
-    if isinstance(data, jpype.JClass):
-        return data.class_
-    if isinstance(data, jpype.JObject):
-        return data.getClass()
     if isinstance(data, str):
+        # Name of a class -- case (A) above.
         return jclass(jimport(data))
+
+    if mode == Mode.JPYPE:
+        start_jvm()
+        if isinstance(data, jpype.JClass):
+            # JPype object representing a static-style class -- case (B) above.
+            return data.class_
+    elif mode == Mode.JEP:
+        if str(type(data.getClass())) == "<class 'jep.PyJClass'>":
+            # Jep object representing a static-style class -- case (B) above.
+            raise ValueError(
+                "Jep does not support Java class objects "
+                + "-- see https://github.com/ninia/jep/issues/405"
+            )
+
+    # A Java object -- case (C) above.
+    if jinstance(data, "java.lang.Object"):
+        return data.getClass()
+
     raise TypeError("Cannot glean class from data of type: " + str(type(data)))
 
 
@@ -400,6 +462,10 @@ def jinstance(obj, jtype) -> bool:
     if isinstance(jtype, str):
         jtype = jimport(jtype)
 
+    if mode == Mode.JEP:
+        return isinstance(obj, jtype.__pytype__)
+
+    assert mode == Mode.JPYPE
     return isinstance(obj, jtype)
 
 
@@ -456,25 +522,54 @@ def jarray(kind, lengths: Sequence):
         lengths = [lengths]
     arraytype = kind
 
-    start_jvm()
+    if mode == Mode.JEP:
+        import jep  # noqa: F401
 
-    # build up the array type
-    kinds = {
-        "b": jpype.JByte,
-        "c": jpype.JChar,
-        "d": jpype.JDouble,
-        "f": jpype.JFloat,
-        "i": jpype.JInt,
-        "j": jpype.JLong,
-        "s": jpype.JShort,
-        "z": jpype.JBoolean,
-    }
-    if arraytype in kinds:
-        arraytype = kinds[arraytype]
-    for _ in range(len(lengths)):
-        arraytype = jpype.JArray(arraytype)
-    # instantiate the n-dimensional array
-    arr = arraytype(lengths[0])
+        if len(lengths) == 1:
+            # Fast case: 1-d array (we can use primitives)
+            arr = jep.jarray(lengths[0], arraytype)
+        else:
+            # Slow case: n-d array (we cannot use primitives)
+            # See https://github.com/ninia/jep/issues/439
+            kinds = {
+                "b": jimport("java.lang.Byte"),
+                "c": jimport("java.lang.Character"),
+                "d": jimport("java.lang.Double"),
+                "f": jimport("java.lang.Float"),
+                "i": jimport("java.lang.Integer"),
+                "j": jimport("java.lang.Long"),
+                "s": jimport("java.lang.Short"),
+                "z": jimport("java.lang.Boolean"),
+            }
+            if arraytype in kinds:
+                arraytype = kinds[arraytype]
+                kind = arraytype
+            # build up the array type
+            for _ in range(len(lengths) - 1):
+                arraytype = jep.jarray(0, arraytype)
+            # instantiate the n-dimensional array
+            arr = jep.jarray(lengths[0], arraytype)
+
+    elif mode == Mode.JPYPE:
+        start_jvm()
+
+        # build up the array type
+        kinds = {
+            "b": jpype.JByte,
+            "c": jpype.JChar,
+            "d": jpype.JDouble,
+            "f": jpype.JFloat,
+            "i": jpype.JInt,
+            "j": jpype.JLong,
+            "s": jpype.JShort,
+            "z": jpype.JBoolean,
+        }
+        if arraytype in kinds:
+            arraytype = kinds[arraytype]
+        for _ in range(len(lengths)):
+            arraytype = jpype.JArray(arraytype)
+        # instantiate the n-dimensional array
+        arr = arraytype(lengths[0])
 
     if len(lengths) > 1:
         for i in range(len(arr)):
